(********************************************************************************)
(*  ocamlcore-api: Access to OCamlCore.org services                             *)
(*                                                                              *)
(*  Copyright (C) 2011, OCamlCore SARL                                          *)
(*                                                                              *)
(*  This library is free software; you can redistribute it and/or modify it     *)
(*  under the terms of the GNU Lesser General Public License as published by    *)
(*  the Free Software Foundation; either version 2.1 of the License, or (at     *)
(*  your option) any later version, with the OCaml static compilation           *)
(*  exception.                                                                  *)
(*                                                                              *)
(*  This library is distributed in the hope that it will be useful, but         *)
(*  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY  *)
(*  or FITNESS FOR A PARTICULAR PURPOSE. See the file COPYING for more          *)
(*  details.                                                                    *)
(*                                                                              *)
(*  You should have received a copy of the GNU Lesser General Public License    *)
(*  along with this library; if not, write to the Free Software Foundation,     *)
(*  Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA               *)
(********************************************************************************)

(** Access account data from the postgresql FusionForge DB
    @author Sylvain Le Gall
  *)

open Lwt

module TZ = CalendarLib.Time_Zone

type t = 
    {
      accnt_id:        int;
      accnt_real_name: string;
      accnt_tz:        TZ.t;
      accnt_lang:      string;
      accnt_unix_id:   int;
      accnt_unix_name: string;
    }

let stub_data: (t OCAStubData.t) = 
  OCAStubData.create ()

let cols, decode_cols = 
  "user_id, user_name, realname, unix_uid, timezone, language",
  (function
     | [Some user_id_str; 
        Some user_name; 
        Some realname;
        Some unix_uid_str;
        Some timezone_str;
        Some language] ->
         {
           accnt_id = 
             OCASql.PGOCaml.int_of_string user_id_str;

           accnt_real_name = 
             if realname <> "" then 
               realname 
             else 
               user_name;

           accnt_tz =
             OCATimezone.of_string timezone_str;

           accnt_lang = 
             language;

           accnt_unix_id =
             OCASql.PGOCaml.int_of_string unix_uid_str;

           accnt_unix_name =
             user_name;
         }
     | lst ->
         failwith 
           (Printf.sprintf 
              "Unexpected user data: %s"
              (String.concat ", "
                 (List.map 
                    (function
                       | Some s -> Printf.sprintf "%S" s
                       | None -> "NULL")
                    lst))))

let list ~ctxt () = 
  match ctxt with 
    | `Live db ->
        OCASql.query db
          ("SELECT "^cols^" FROM users WHERE status = 'A'")
          []
          >>= 
          Lwt_list.map_s (fun e -> return (decode_cols e))
    | `Stub stub_id ->
        OCAStubData.get stub_data stub_id (fun accnt -> true)

let of_id ~ctxt id = 
  match ctxt with 
    | `Live db ->
        begin
          OCASql.query db
            ("SELECT "^cols^" FROM users WHERE status = 'A' AND user_id = $1")
            [Some (OCASql.PGOCaml.string_of_int id)]
          >>= function
            | res :: _ ->
                return (decode_cols res)
            | [] ->
                fail Not_found
        end

    | `Stub stub_id ->
        OCAStubData.get_one stub_data stub_id (fun t -> t.accnt_id = id)

let of_unix_id ~ctxt uid = 
  match ctxt with 
    | `Live db ->
        begin
          OCASql.query db
            ("SELECT "^cols^" FROM users WHERE status = 'A' AND unix_uid = $1")
            [Some (OCASql.PGOCaml.string_of_int uid)]
          >>= function
            | res :: _ ->
                return (decode_cols res)
            | [] ->
                fail Not_found
        end

    | `Stub stub_id ->
        OCAStubData.get_one stub_data stub_id (fun t -> t.accnt_unix_id = uid)

let of_unix_name ~ctxt unix_name =
  match ctxt with 
    | `Live db ->
        begin
          OCASql.query db
            ("SELECT "^cols^" FROM users WHERE status = 'A' AND user_name = $1")
            [Some unix_name]
          >>= function
            | res :: _ ->
                return (decode_cols res)
            | [] ->
                fail Not_found
        end

    | `Stub stub_id ->
        OCAStubData.get_one stub_data stub_id 
          (fun t -> t.accnt_unix_name = unix_name)

let to_string t =
  Printf.sprintf
    "%s (id: %d; tz: %s; lang: %s; unix id: %d; unix name: %s)"
    t.accnt_real_name
    t.accnt_id
    (OCATimezone.to_string t.accnt_tz)
    t.accnt_lang
    t.accnt_unix_id
    t.accnt_unix_name
